MODULE usrdef_zgr
   !!======================================================================
   !!                   ***  MODULE  usrdef_zgr  ***
   !!
   !!                   ===  WAD_TEST_CASES case  ===
   !!
   !! Ocean domain : user defined vertical coordinate system 
   !!======================================================================
   !! History :  4.0  ! 2016-08  (G. Madec, S. Flavoni)  Original code
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   usr_def_zgr   : user defined vertical coordinate system (required)
   !!       zgr_z1d   : reference 1D z-coordinate 
   !!---------------------------------------------------------------------
   USE oce            ! ocean variables
   USE dom_oce ,  ONLY: ln_zco, ln_zps, ln_sco   ! ocean space and time domain
   USE dom_oce ,  ONLY: mi0, mi1, nimpp, njmpp,  &
                      & mj0, mj1, glamt, gphit   ! ocean space and time domain
   USE usrdef_nam     ! User defined : namelist variables
   USE wet_dry ,  ONLY: rn_wdmin1, rn_wdmin2, rn_wdld, ht_wd
   !
   USE in_out_manager ! I/O manager
   USE lbclnk         ! ocean lateral boundary conditions (or mpp link)
   USE lib_mpp        ! distributed memory computing library
   USE wrk_nemo       ! Memory allocation
   USE timing         ! Timing

   IMPLICIT NONE
   PRIVATE

   PUBLIC   usr_def_zgr   ! called by domzgr.F90

  !! * Substitutions
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 4.0 , NEMO Consortium (2016)
   !! $Id$
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS             

   SUBROUTINE usr_def_zgr( ld_zco  , ld_zps  , ld_sco  , ld_isfcav,    &   ! type of vertical coordinate
      &                    pdept_1d, pdepw_1d, pe3t_1d , pe3w_1d  ,    &   ! 1D reference vertical coordinate
      &                    pdept , pdepw ,                             &   ! 3D t & w-points depth
      &                    pe3t  , pe3u  , pe3v , pe3f ,               &   ! vertical scale factors
      &                    pe3w  , pe3uw , pe3vw,                      &   !     -      -      -
      &                    k_top  , k_bot    )                             ! top & bottom ocean level
      !!---------------------------------------------------------------------
      !!              ***  ROUTINE usr_def_zgr  ***
      !!
      !! ** Purpose :   User defined the vertical coordinates
      !!
      !!----------------------------------------------------------------------
      LOGICAL                   , INTENT(out) ::   ld_zco, ld_zps, ld_sco      ! vertical coordinate flags
      LOGICAL                   , INTENT(out) ::   ld_isfcav                   ! under iceshelf cavity flag
      REAL(wp), DIMENSION(:)    , INTENT(out) ::   pdept_1d, pdepw_1d          ! 1D grid-point depth     [m]
      REAL(wp), DIMENSION(:)    , INTENT(out) ::   pe3t_1d , pe3w_1d           ! 1D grid-point depth     [m]
      REAL(wp), DIMENSION(:,:,:), INTENT(out) ::   pdept, pdepw                ! grid-point depth        [m]
      REAL(wp), DIMENSION(:,:,:), INTENT(out) ::   pe3t , pe3u , pe3v , pe3f   ! vertical scale factors  [m]
      REAL(wp), DIMENSION(:,:,:), INTENT(out) ::   pe3w , pe3uw, pe3vw         ! i-scale factors 
      INTEGER , DIMENSION(:,:)  , INTENT(out) ::   k_top, k_bot                ! first & last ocean level
      !
      INTEGER  ::   ji, jj, jk        ! dummy indices
      INTEGER  ::   ik                ! local integers
      REAL(wp) ::   zfact, z1_jpkm1   ! local scalar
      REAL(wp) ::   ze3min            ! local scalar
      REAL(wp) ::   zi, zj, zbathy    ! local scalar
      REAL(wp) ::   ztmpu, ztmpv, ztmpf, ztmpu1, ztmpv1, ztmpf1
      REAL(wp), DIMENSION(jpi,jpj) ::   zht, zhu, zhv, z2d   ! 2D workspace
      !!----------------------------------------------------------------------
      !
      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) 'usr_def_zgr : WAD_TEST_CASES configuration (s-coordinate closed box ocean without cavities)'
      IF(lwp) WRITE(numout,*) '~~~~~~~~~~~'
      !
      !
      ! type of vertical coordinate
      ! ---------------------------
      ! set in usrdef_nam.F90 by reading the namusr_def namelist only ln_zco
      ln_zco    = .FALSE.      ! z-partial-step coordinate
      ln_zps    = .FALSE.      ! z-partial-step coordinate
      ln_sco    = .TRUE.       ! s-coordinate
      ld_isfcav = .FALSE.      ! ISF Ice Shelves Flag
      !
      !
      ! Build the vertical coordinate system
      ! ------------------------------------
      !
      !                       !==  UNmasked meter bathymetry  ==!
      !
!
      zbathy=10.0
      IF( cn_cfg == 'wad' ) THEN
         SELECT CASE ( nn_cfg )
            !                                        ! ====================
            CASE ( 1 )                               ! WAD 1 configuration
               !                                     ! ====================
               !
               IF(lwp) WRITE(numout,*)
               IF(lwp) WRITE(numout,*) 'usr_def_zgr (WAD) : Closed box with EW linear bottom slope'
               IF(lwp) WRITE(numout,*) '~~~~~~~~~~'
               !
               zht = 1.5_wp
               DO ji = 1, jpi
                 zi = MIN((glamt(ji,1) - 10.0)/40.0, 1.0 )
                 zht(ji,:) = MAX(zbathy*zi, 1.5) 
                 IF(lwp)write(numout,*) 'ZDTA ',ji,zi,zht(ji,1)
               END DO
               zht(mi0(1):mi1(1),:) = -4._wp
               zht(mi0(jpiglo):mi1(jpiglo),:) = -4._wp
               zht(:,mj0(1):mj1(1)) = -4._wp
               zht(:,mj0(jpjglo):mj1(jpjglo)) = -4._wp
               !                                     ! ====================
            CASE ( 2, 3 )                            ! WAD 2 or 3  configuration
               !                                     ! ====================
               !
               IF(lwp) WRITE(numout,*)
               IF(lwp) WRITE(numout,*) 'usr_def_zgr (WAD) : Parobolic EW channel'
               IF(lwp) WRITE(numout,*) '~~~~~~~~~~'
               !
               DO ji = 1, jpi
                 zi = MAX(1.0-((glamt(ji,1)-25.0)**2)/484.0, 0.0 )
                 zi = MAX(1.0-((glamt(ji,1)-25.0)**2)/484.0, -2.0 )
                 zht(ji,:) = MAX(zbathy*zi, -20.0)
                 IF(lwp)write(numout,*) 'ZDTA ',ji,zi,zht(ji,1)
               END DO
               zht(mi0(1):mi1(2),:) = -4._wp
               zht(mi0(jpiglo-1):mi1(jpiglo),:) = -4._wp
               zht(:,mj0(1):mj1(1)) = -4._wp
               zht(:,mj0(jpjglo):mj1(jpjglo)) = -4._wp
               !                                     ! ====================
            CASE ( 4 )                               ! WAD 4 configuration
               !                                     ! ====================
               !
               IF(lwp) WRITE(numout,*)
               IF(lwp) WRITE(numout,*) 'usr_def_zgr (WAD) : Parobolic bowl'
               IF(lwp) WRITE(numout,*) '~~~~~~~~~~'
               !
               DO ji = 1, jpi
                 zi = MAX(1.0-((glamt(ji,1)-25.0)**2)/484.0, -2.0 )
               DO jj = 1, jpj
                 zj = MAX(1.0-((gphit(1,jj)-17.0)**2)/196.0, -2.0 )
                 zht(ji,jj) = MAX(zbathy*zi*zj, -2.0)
               END DO
                 IF(lwp)write(numout,*) 'ZDTA ',ji,zi,zht(ji,1)
               END DO
               zht(mi0(1):mi1(1),:) = -4._wp
               zht(mi0(jpiglo-1):mi1(jpiglo),:) = -4._wp
               zht(:,mj0(1):mj1(1)) = -4._wp
               zht(:,mj0(jpjglo):mj1(jpjglo)) = -4._wp
               !                                    ! ===========================
            CASE ( 5 )                              ! WAD 5 configuration
               !                                    ! ====================
               !
               IF(lwp) WRITE(numout,*)
               IF(lwp) WRITE(numout,*) 'usr_def_zgr (WAD) : Double slope with shelf'
               IF(lwp) WRITE(numout,*) '~~~~~~~~~~'
               !
               DO ji = 1, jpi
                 zi = MIN(glamt(ji,1)/45.0, 1.0 )
                 zht(ji,:) = MAX(zbathy*zi, 0.5)
                 IF( glamt(ji,1) >= 46.0 ) THEN
                   zht(ji,:) = 10.0
                 ELSE IF( glamt(ji,1) >= 20.0 .AND. glamt(ji,1) < 46.0 ) THEN
                   zi = 7.5/25.
                   zht(ji,:) = MAX(10. - zi*(47.-glamt(ji,1)),2.5)
                 ELSE IF( glamt(ji,1) >= 15.0 .AND. glamt(ji,1) < 20.0 ) THEN
                   zht(ji,:) = 2.5
                 ELSE IF( glamt(ji,1) >= 4.0 .AND. glamt(ji,1) < 15.0 ) THEN
                   zi = 2.0/11.0
                   zht(ji,:) = MAX(2.5 - zi*(16.0-glamt(ji,1)), 0.5)
                 ELSE IF( glamt(ji,1) >= 1.0 .AND. glamt(ji,1) < 4.0 ) THEN
                   zht(ji,:) = 0.5
                 ENDIF
                 IF(lwp)write(numout,*) 'ZDTA ',ji,zi,zht(ji,1)
               END DO
               !                                    ! ===========================
               zht(mi0(1):mi1(1),:) = -4._wp
               zht(mi0(jpiglo):mi1(jpiglo),:) = -4._wp
               zht(:,mj0(1):mj1(1)) = -4._wp
               zht(:,mj0(jpjglo):mj1(jpjglo)) = -4._wp
               !                                    ! ===========================
            CASE ( 6 )                              ! WAD 6 configuration
               !                                    ! ====================
               !
               IF(lwp) WRITE(numout,*)
               IF(lwp) WRITE(numout,*) 'usr_def_zgr (WAD) : Parabolic channel with gaussian ridge'
               IF(lwp) WRITE(numout,*) '~~~~~~~~~~'
               !
               DO ji = 1, jpi
                 zi = MAX(1.0-((glamt(ji,1)-25.0)**2)/484.0, -2.0 )
                 zj = 0.95*MAX(EXP(-1.0*((glamt(ji,1)-25.0)**2)/32.0) , 0.0 )
                 zht(ji,:) = MAX(zbathy*(zi-zj), -2.0)
                 IF(lwp)write(numout,*) 'ZDTA ',ji,zi,zht(ji,1)
               END DO
               zht(mi0(1):mi1(1),:) = -4._wp
               zht(mi0(jpiglo):mi1(jpiglo),:) = -4._wp
               zht(:,mj0(1):mj1(1)) = -4._wp
               zht(:,mj0(jpjglo):mj1(jpjglo)) = -4._wp
               !                                    ! ===========================
            CASE DEFAULT
               !                                    ! ===========================
               WRITE(ctmp1,*) 'WAD test with a ', nn_cfg,' option is not coded'
               !
               CALL ctl_stop( ctmp1 )
               !
         END SELECT
      END IF
               !
      !
      ! at u-point: averaging zht
      DO ji = 1, jpim1
         zhu(ji,:) = 0.5_wp * ( zht(ji,:) + zht(ji+1,:) )
      END DO
      IF(lwp) write(numout,*) 'E3a zu ', MINVAL(zht(:,:)), MINVAL(zhu(:,:)), MINVAL(zhv(:,:))
      CALL lbc_lnk( zhu, 'U', 1. )     ! boundary condition: this mask the surrounding grid-points
      !                                ! ==>>>  set by hand non-zero value on first/last columns & rows 
      IF(lwp) write(numout,*) 'E3b zu ', MINVAL(zht(:,:)), MINVAL(zhu(:,:)), MINVAL(zhv(:,:))
      DO ji = mi0(1), mi1(1)              ! first row of global domain only
         zhu(ji,:) = zht(1,:)
      END DO
       DO ji = mi0(jpiglo), mi1(jpiglo)   ! last  row of global domain only
         zhu(ji,:) = zht(jpi,:)
      END DO
      IF(lwp) write(numout,*) 'E3c zu ', MINVAL(zht(:,:)), MINVAL(zhu(:,:)), MINVAL(zhv(:,:))
      ! at v-point: averaging zht
      zhv = 0._wp
      DO jj = 1, jpjm1
         zhv(:,jj) = 0.5_wp * ( zht(:,jj) + zht(:,jj+1) )
      END DO
      CALL lbc_lnk( zhv, 'V', 1. )     ! boundary condition: this mask the surrounding grid-points
      IF(lwp) write(numout,*) 'E3d zu ', MINVAL(zht(:,:)), MINVAL(zhu(:,:)), MINVAL(zhv(:,:))
      !avoid the zero depth on T- (U-,V-,F-) points
        DO jj = 1, jpj
          DO ji = 1, jpi
            IF(ABS(zht(ji,jj)) < rn_wdmin1) &
              & zht(ji,jj) = SIGN(1._wp, zht(ji,jj)) * rn_wdmin1
            IF(ABS(zhu(ji,jj)) < rn_wdmin1) &
              & zhu(ji,jj) = SIGN(1._wp, zhu(ji,jj)) * rn_wdmin1
            IF(ABS(zhv(ji,jj)) < rn_wdmin1) &
              & zhv(ji,jj) = SIGN(1._wp, zhv(ji,jj)) * rn_wdmin1
          END DO
        END DO
      IF(lwp) write(numout,*) 'E3e zu ', MINVAL(zht(:,:)), MINVAL(zhu(:,:)), MINVAL(zhv(:,:))
      !     
      CALL zgr_z1d( pdept_1d, pdepw_1d, pe3t_1d , pe3w_1d )   ! Reference z-coordinate system
      !
      !
      !                       !==  top masked level bathymetry  ==!  (all coordinates)
      !
      ! no ocean cavities : top ocean level is ONE, except over land
      ! the ocean basin surrounnded by land (1 grid-point) set through lbc_lnk call as jperio=0 
      z2d(:,:) = 1._wp                    ! surface ocean is the 1st level
      z2d(mi0(1):mi1(1),:) = 0._wp
      z2d(mi0(jpiglo):mi1(jpiglo),:) = 0._wp
      z2d(:,mj0(1):mj1(1)) = 0._wp
      z2d(:,mj0(jpjglo):mj1(jpjglo)) = 0._wp





      CALL lbc_lnk( z2d, 'T', 1. )        ! closed basin since jperio = 0 (see userdef_nam.F90)
      k_top(:,:) = NINT( z2d(:,:) )
      !
      !                              
      !
      IF ( ln_sco ) THEN      !==  s-coordinate  ==!   (terrain-following coordinate)
         !
         ht_wd = zht
         k_bot(:,:) = jpkm1 * k_top(:,:)  !* bottom ocean = jpk-1 (here use k_top as a land mask)
         DO jj = 1, jpj
            DO ji = 1, jpi
              IF( zht(ji,jj) <= -(rn_wdld - rn_wdmin2)) THEN
                !WRITE(numout,*) 'KBOT ',ji,jj,zht(ji,jj),rn_wdld,rn_wdmin2
                k_bot(ji,jj) = 0
                zht(ji,jj) = rn_wdmin1 * REAL( jpkm1 , wp) * 0.5_wp
              ELSEIF(zht(ji,jj) <= rn_wdmin1) THEN
                k_bot(ji,jj) = 2
                zht(ji,jj) = rn_wdmin1 * REAL( jpkm1 , wp) * 0.5_wp
                !WRITE(numout,*) 'KBOT1 ',ji,jj,zht(ji,jj),rn_wdmin1
              ENDIF
           END DO
         END DO
      DO ji = 1, jpim1
         zhu(ji,:) = 0.5_wp * ( zht(ji,:) + zht(ji+1,:) )
      END DO
      IF(lwp) write(numout,*) 'E3f zu ', MINVAL(zht(:,:)), MINVAL(zhu(:,:)), MINVAL(zhv(:,:))
      !                                ! ==>>>  set by hand non-zero value on
      !                                first/last columns & rows 
      IF(lwp) write(numout,*) 'E3f Zu ', MINVAL(zht(:,:)), MINVAL(zhu(:,:)), MINVAL(zhv(:,:))
      CALL lbc_lnk( zhu, 'U', 1. )     ! boundary condition: this mask the surrounding grid-points
      IF(lwp) write(numout,*) 'E3F zu ', MINVAL(zht(:,:)), MINVAL(zhu(:,:)), MINVAL(zhv(:,:))
      DO ji = mi0(1), mi1(1)              ! first row of global domain only
         zhu(ji,:) = zht(ji,:)
      END DO
      DO ji = mi0(jpiglo), mi1(jpiglo)   ! last  row of global domain only
         zhu(ji,:) = zht(ji,:)
      END DO
      ! at v-point: averaging zht
      IF(lwp) write(numout,*) 'E3g zu ', MINVAL(zht(:,:)), MINVAL(zhu(:,:)), MINVAL(zhv(:,:))
      zhv = 0._wp
      DO jj = 1, jpjm1
         zhv(:,jj) = 0.5_wp * ( zht(:,jj) + zht(:,jj+1) )
      END DO
      IF(lwp) write(numout,*) 'E3h zu ', MINVAL(zht(:,:)), MINVAL(zhu(:,:)), MINVAL(zhv(:,:))
      CALL lbc_lnk( zhv, 'V', 1. )
      IF(lwp) write(numout,*) 'E3H zu ', MINVAL(zht(:,:)), MINVAL(zhu(:,:)), MINVAL(zhv(:,:))
      DO jj = mj0(1), mj1(1)   ! first  row of global domain only
         zhv(:,jj) = zht(:,jj)
      END DO
      DO jj = mj0(jpjglo), mj1(jpjglo)   ! last  row of global domain only
         zhv(:,jj) = zht(:,jj)
      END DO
         !
         !                                !* terrain-following coordinate with e3.(k)=cst)
         !                                !  OVERFLOW case : identical with j-index (T=V, U=F)
         z1_jpkm1 = 1._wp / REAL( jpkm1 , wp)
         DO jk = 1, jpk
                  pdept(:,:,jk) = zht(:,:) * z1_jpkm1 * ( REAL( jk   , wp ) - 0.5_wp )
                  pdepw(:,:,jk) = zht(:,:) * z1_jpkm1 * ( REAL( jk-1 , wp )          )
                  pe3t (:,:,jk) = zht(:,:) * z1_jpkm1
                  pe3u (:,:,jk) = zhu(:,:) * z1_jpkm1
                  pe3v (:,:,jk) = zhv(:,:) * z1_jpkm1
                  pe3f (:,:,jk) = zhu(:,:) * z1_jpkm1
                  pe3w (:,:,jk) = zht(:,:) * z1_jpkm1
                  pe3uw(:,:,jk) = zhu(:,:) * z1_jpkm1
                  pe3vw(:,:,jk) = zhv(:,:) * z1_jpkm1
         END DO      
         DO jj = 1, jpjm1
            DO ji = 1, jpim1
               ztmpu  = zht(ji,jj)+zht(ji+1,jj)
               ztmpv  = zht(ji,jj)+zht(ji,jj+1)
               ztmpf  = zht(ji,jj)+zht(ji+1,jj)+zht(ji,jj+1)+zht(ji+1,jj+1)
               ztmpu1 = zht(ji,jj)*zht(ji+1,jj)
               ztmpv1 = zht(ji,jj)*zht(ji,jj+1)
               ztmpf1 = MIN(zht(ji,jj), zht(ji+1,jj), zht(ji,jj+1), zht(ji+1,jj+1)) * &
                      & MAX(zht(ji,jj), zht(ji+1,jj), zht(ji,jj+1), zht(ji+1,jj+1))
               IF(  (ztmpu1 < 0._wp.OR.ABS(ztmpu) < rn_wdmin1) ) THEN
                 pe3u (ji,jj,:) = 0.5_wp* ( pe3t(ji,jj,:) + pe3t(ji+1,jj,:) )
                 pe3uw(ji,jj,:) = 0.5_wp* ( pe3w(ji,jj,:) + pe3w(ji+1,jj,:) )
               ENDIF
               IF(  (ztmpv1 < 0._wp.OR.ABS(ztmpv) < rn_wdmin1) ) THEN
                 pe3v (ji,jj,:) = 0.5_wp* ( pe3t(ji,jj,:) + pe3t(ji,jj+1,:) )
                 pe3vw(ji,jj,:) = 0.5_wp* ( pe3w(ji,jj,:) + pe3w(ji,jj+1,:) )
               ENDIF
               IF(  (ztmpf1 < 0._wp.OR.ABS(ztmpf) < rn_wdmin1) ) THEN
                 pe3f (ji,jj,:) = 0.25_wp* ( pe3t(ji,jj,:)   + pe3t(ji+1,jj,:) &
                                &          + pe3t(ji,jj+1,:) + pe3t(ji+1,jj+1,:) )
               ENDIF
                 pe3f (ji,jj,:) = 0.25_wp* ( pe3t(ji,jj,:)   + pe3t(ji+1,jj,:) &
                                &          + pe3t(ji,jj+1,:) + pe3t(ji+1,jj+1,:) )
           END DO
         END DO
         CALL lbc_lnk( pe3u , 'U', 1. )
         CALL lbc_lnk( pe3uw, 'U', 1. )
         CALL lbc_lnk( pe3v , 'V', 1. )
         CALL lbc_lnk( pe3vw, 'V', 1. )
         CALL lbc_lnk( pe3f , 'F', 1. )
      WHERE( pe3t (:,:,:) == 0._wp )   pe3t (:,:,:) = 1._wp
      WHERE( pe3u (:,:,:) == 0._wp )   pe3u (:,:,:) = 1._wp
      WHERE( pe3v (:,:,:) == 0._wp )   pe3v (:,:,:) = 1._wp
      WHERE( pe3f (:,:,:) == 0._wp )   pe3f (:,:,:) = 1._wp
      WHERE( pe3w (:,:,:) == 0._wp )   pe3w (:,:,:) = 1._wp
      WHERE( pe3uw(:,:,:) == 0._wp )   pe3uw(:,:,:) = 1._wp
      WHERE( pe3vw(:,:,:) == 0._wp )   pe3vw(:,:,:) = 1._wp
      IF(lwp) write(numout,*) 'E3 TU ',SUM(0._wp/pe3t(:,:,:)), MINVAL(pe3t(:,:,:)), rn_wdmin1 ; CALL FLUSH(numout)
      IF(lwp) write(numout,*) 'E3 tu ',SUM(0._wp/pe3u(:,:,:)), MINVAL(pe3u(:,:,:))
      IF(lwp) write(numout,*) 'E3 tv ',SUM(0._wp/pe3v(:,:,:)), MINVAL(pe3v(:,:,:))
      IF(lwp) write(numout,*) 'E3 tf ',SUM(0._wp/pe3f(:,:,:)), MINVAL(pe3f(:,:,:))
      IF(lwp) write(numout,*) 'E3 zu ', MINVAL(zht(:,:)), MINVAL(zhu(:,:)), MINVAL(zhv(:,:))
      ENDIF
      !
      !
      !
   END SUBROUTINE usr_def_zgr


   SUBROUTINE zgr_z1d( pdept_1d, pdepw_1d, pe3t_1d , pe3w_1d )   ! 1D reference vertical coordinate
      !!----------------------------------------------------------------------
      !!                   ***  ROUTINE zgr_z1d  ***
      !!
      !! ** Purpose :   set the depth of model levels and the resulting 
      !!      vertical scale factors.
      !!
      !! ** Method  :   1D z-coordinate system (use in all type of coordinate)
      !!       The depth of model levels is set from dep(k), an analytical function:
      !!                   w-level: depw_1d  = dep(k)
      !!                   t-level: dept_1d  = dep(k+0.5)
      !!       The scale factors are the discrete derivative of the depth:
      !!                   e3w_1d(jk) = dk[ dept_1d ] 
      !!                   e3t_1d(jk) = dk[ depw_1d ]
      !!
      !!            ===    Here constant vertical resolution   ===
      !!
      !! ** Action  : - pdept_1d, pdepw_1d : depth of T- and W-point (m)
      !!              - pe3t_1d , pe3w_1d  : scale factors at T- and W-levels (m)
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(:), INTENT(out) ::   pdept_1d, pdepw_1d   ! 1D grid-point depth        [m]
      REAL(wp), DIMENSION(:), INTENT(out) ::   pe3t_1d , pe3w_1d    ! 1D vertical scale factors  [m]
      !
      INTEGER  ::   jk       ! dummy loop indices
      REAL(wp) ::   zt, zw   ! local scalar
      !!----------------------------------------------------------------------
      !
      IF(lwp) THEN                         ! Parameter print
         WRITE(numout,*)
         WRITE(numout,*) '    zgr_z1d : Reference vertical z-coordinates: uniform dz = ', rn_dz
         WRITE(numout,*) '    ~~~~~~~'
      ENDIF
      !
      ! Reference z-coordinate (depth - scale factor at T- and W-points)   ! Madec & Imbard 1996 function
      ! ----------------------
      DO jk = 1, jpk
         zw = REAL( jk , wp )
         zt = REAL( jk , wp ) + 0.5_wp
         pdepw_1d(jk) =    rn_dz *   REAL( jk-1 , wp )
         pdept_1d(jk) =    rn_dz * ( REAL( jk-1 , wp ) + 0.5_wp )
         pe3w_1d (jk) =    rn_dz
         pe3t_1d (jk) =    rn_dz
      END DO
      !
      IF(lwp) THEN                        ! control print
         WRITE(numout,*)
         WRITE(numout,*) '              Reference 1D z-coordinate depth and scale factors:'
         WRITE(numout, "(9x,' level  gdept_1d  gdepw_1d  e3t_1d   e3w_1d  ')" )
         WRITE(numout, "(10x, i4, 4f9.2)" ) ( jk, pdept_1d(jk), pdepw_1d(jk), pe3t_1d(jk), pe3w_1d(jk), jk = 1, jpk )
      ENDIF
      !
   END SUBROUTINE zgr_z1d
   
   !!======================================================================
END MODULE usrdef_zgr
